import { drizzle } from 'drizzle-orm/node-postgres';
import { migrate } from 'drizzle-orm/node-postgres/migrator';
import { Pool } from 'pg';
import * as dotenv from 'dotenv';
import * as path from 'path';
import * as schema from '../schema';

/**
 * Script to run database migrations
 * 
 * This script:
 * 1. Establishes a connection to the PostgreSQL database
 * 2. Creates a Drizzle ORM instance
 * 3. Runs all pending migrations
 * 
 * It can be used:
 * - As a standalone script: `node dist/database/migrations/migrate.js`
 * - Integrated with NestJS application lifecycle in database.service.ts
 */

// Load environment variables
dotenv.config();

export const runMigrations = async (options?: { migrationsFolder?: string }) => {
  // First try to get the full DATABASE_URL
  let connectionString = process.env.DATABASE_URL;

  // If DATABASE_URL is not provided, construct it from individual variables
  if (!connectionString) {
    const password = process.env.POSTGRES_PASSWORD || 'postgres';
    const username = process.env.POSTGRES_USER || 'postgres';
    const host = process.env.POSTGRES_HOST || 'localhost';
    const port = Number(process.env.POSTGRES_PORT || 5432);
    const database = process.env.POSTGRES_DB || 'groupmaker';

    connectionString = `postgres://${username}:${password}@${host}:${port}/${database}`;
  }

  // Create the PostgreSQL pool
  const pool = new Pool({
    connectionString,
  });

  // Create the Drizzle ORM instance
  const db = drizzle(pool, { schema });

  console.log('Running migrations...');

  try {
    // Test the connection
    const client = await pool.connect();
    try {
      await client.query('SELECT NOW()');
      console.log('Database connection established successfully');
    } finally {
      client.release();
    }

    // Determine migrations folder path
    const migrationsFolder = options?.migrationsFolder || path.join(__dirname, 'sql');
    console.log(`Using migrations folder: ${migrationsFolder}`);

    // Run migrations
    await migrate(db, { migrationsFolder });

    console.log('Migrations completed successfully');
    return { success: true };
  } catch (error) {
    console.error('Migration failed');
    console.error(error);
    return { success: false, error };
  } finally {
    await pool.end();
    console.log('Database connection closed');
  }
};

// Run migrations if this script is executed directly
if (require.main === module) {
  runMigrations()
    .then(result => {
      if (!result.success) {
        process.exit(1);
      }
    })
    .catch(err => {
      console.error('Migration failed');
      console.error(err);
      process.exit(1);
    });
}
