import { Test, TestingModule } from '@nestjs/testing';
import { ConfigService } from '@nestjs/config';
import { UnauthorizedException } from '@nestjs/common';
import { AuthController } from './auth.controller';
import { AuthService } from '../services/auth.service';

describe('AuthController', () => {
  let controller: AuthController;
  let authService: AuthService;
  let configService: ConfigService;

  const mockAuthService = {
    validateGithubUser: jest.fn(),
    generateTokens: jest.fn(),
    refreshTokens: jest.fn(),
  };

  const mockConfigService = {
    get: jest.fn(),
  };

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      controllers: [AuthController],
      providers: [
        { provide: AuthService, useValue: mockAuthService },
        { provide: ConfigService, useValue: mockConfigService },
      ],
    }).compile();

    controller = module.get<AuthController>(AuthController);
    authService = module.get<AuthService>(AuthService);
    configService = module.get<ConfigService>(ConfigService);
  });

  afterEach(() => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(controller).toBeDefined();
  });

  describe('githubAuth', () => {
    it('should be defined', () => {
      expect(controller.githubAuth).toBeDefined();
    });
  });

  describe('githubAuthCallback', () => {
    it('should redirect to frontend with tokens', async () => {
      const req = {
        user: { id: 'user1', name: 'Test User' },
      };
      const res = {
        redirect: jest.fn(),
      };
      const tokens = {
        accessToken: 'access-token',
        refreshToken: 'refresh-token',
      };
      const frontendUrl = 'http://localhost:3001';
      const expectedRedirectUrl = `${frontendUrl}/auth/callback?accessToken=${tokens.accessToken}&refreshToken=${tokens.refreshToken}`;

      mockAuthService.generateTokens.mockResolvedValue(tokens);
      mockConfigService.get.mockReturnValue(frontendUrl);

      await controller.githubAuthCallback(req as any, res as any);

      expect(mockAuthService.generateTokens).toHaveBeenCalledWith('user1');
      expect(mockConfigService.get).toHaveBeenCalledWith('FRONTEND_URL');
      expect(res.redirect).toHaveBeenCalledWith(expectedRedirectUrl);
    });

    it('should throw UnauthorizedException if user is not provided', async () => {
      const req = {};
      const res = {
        redirect: jest.fn(),
      };

      await expect(controller.githubAuthCallback(req as any, res as any)).rejects.toThrow(
        UnauthorizedException,
      );
      expect(res.redirect).not.toHaveBeenCalled();
    });
  });

  describe('refreshTokens', () => {
    it('should refresh tokens', async () => {
      const user = {
        id: 'user1',
        refreshToken: 'refresh-token',
      };
      const tokens = {
        accessToken: 'new-access-token',
        refreshToken: 'new-refresh-token',
      };

      mockAuthService.refreshTokens.mockResolvedValue(tokens);

      const result = await controller.refreshTokens(user);

      expect(mockAuthService.refreshTokens).toHaveBeenCalledWith('user1', 'refresh-token');
      expect(result).toEqual(tokens);
    });
  });

  describe('getProfile', () => {
    it('should return user profile', () => {
      const user = { id: 'user1', name: 'Test User' };

      const result = controller.getProfile(user);

      expect(result).toEqual(user);
    });
  });
});