import { Test, TestingModule } from '@nestjs/testing';
import { JwtService } from '@nestjs/jwt';
import { ConfigService } from '@nestjs/config';
import { AuthService } from './auth.service';
import { UsersService } from '../../users/services/users.service';
import { UnauthorizedException } from '@nestjs/common';

describe('AuthService', () => {
  let service: AuthService;
  let usersService: UsersService;
  let jwtService: JwtService;
  let configService: ConfigService;

  const mockUsersService = {
    findByGithubId: jest.fn(),
    findById: jest.fn(),
    create: jest.fn(),
    update: jest.fn(),
  };

  const mockJwtService = {
    signAsync: jest.fn(),
    verifyAsync: jest.fn(),
  };

  const mockConfigService = {
    get: jest.fn(),
  };

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        AuthService,
        { provide: UsersService, useValue: mockUsersService },
        { provide: JwtService, useValue: mockJwtService },
        { provide: ConfigService, useValue: mockConfigService },
      ],
    }).compile();

    service = module.get<AuthService>(AuthService);
    usersService = module.get<UsersService>(UsersService);
    jwtService = module.get<JwtService>(JwtService);
    configService = module.get<ConfigService>(ConfigService);
  });

  afterEach(() => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(service).toBeDefined();
  });

  describe('validateGithubUser', () => {
    it('should create a new user if one does not exist', async () => {
      const githubId = 'github123';
      const email = 'test@example.com';
      const name = 'Test User';
      const avatarUrl = 'https://example.com/avatar.jpg';
      const newUser = { id: 'user1', githubId, name, avatar: avatarUrl };

      mockUsersService.findByGithubId.mockResolvedValue(null);
      mockUsersService.create.mockResolvedValue(newUser);

      const result = await service.validateGithubUser(githubId, email, name, avatarUrl);

      expect(mockUsersService.findByGithubId).toHaveBeenCalledWith(githubId);
      expect(mockUsersService.create).toHaveBeenCalledWith({
        githubId,
        name,
        avatar: avatarUrl,
        metadata: { email },
      });
      expect(result).toEqual(newUser);
    });

    it('should return an existing user if one exists', async () => {
      const githubId = 'github123';
      const email = 'test@example.com';
      const name = 'Test User';
      const avatarUrl = 'https://example.com/avatar.jpg';
      const existingUser = { id: 'user1', githubId, name, avatar: avatarUrl };

      mockUsersService.findByGithubId.mockResolvedValue(existingUser);

      const result = await service.validateGithubUser(githubId, email, name, avatarUrl);

      expect(mockUsersService.findByGithubId).toHaveBeenCalledWith(githubId);
      expect(mockUsersService.create).not.toHaveBeenCalled();
      expect(result).toEqual(existingUser);
    });
  });

  describe('generateTokens', () => {
    it('should generate access and refresh tokens', async () => {
      const userId = 'user1';
      const accessToken = 'access-token';
      const refreshToken = 'refresh-token';
      const refreshExpiration = '7d';
      const refreshSecret = 'refresh-secret';

      mockJwtService.signAsync.mockResolvedValueOnce(accessToken);
      mockJwtService.signAsync.mockResolvedValueOnce(refreshToken);
      mockConfigService.get.mockReturnValueOnce(refreshExpiration);
      mockConfigService.get.mockReturnValueOnce(refreshSecret);

      const result = await service.generateTokens(userId);

      expect(mockJwtService.signAsync).toHaveBeenCalledWith({ sub: userId });
      expect(mockJwtService.signAsync).toHaveBeenCalledWith(
        { sub: userId, isRefreshToken: true },
        {
          expiresIn: refreshExpiration,
          secret: refreshSecret,
        },
      );
      expect(result).toEqual({
        accessToken,
        refreshToken,
      });
    });
  });

  describe('refreshTokens', () => {
    it('should refresh tokens if refresh token is valid', async () => {
      const userId = 'user1';
      const refreshToken = 'valid-refresh-token';
      const newAccessToken = 'new-access-token';
      const newRefreshToken = 'new-refresh-token';
      const payload = { sub: userId, isRefreshToken: true };

      mockJwtService.verifyAsync.mockResolvedValue(payload);
      mockJwtService.signAsync.mockResolvedValueOnce(newAccessToken);
      mockJwtService.signAsync.mockResolvedValueOnce(newRefreshToken);

      const result = await service.refreshTokens(userId, refreshToken);

      expect(mockJwtService.verifyAsync).toHaveBeenCalledWith(refreshToken, {
        secret: undefined,
      });
      expect(result).toEqual({
        accessToken: newAccessToken,
        refreshToken: newRefreshToken,
      });
    });

    it('should throw UnauthorizedException if refresh token is invalid', async () => {
      const userId = 'user1';
      const refreshToken = 'invalid-refresh-token';

      mockJwtService.verifyAsync.mockRejectedValue(new Error('Invalid token'));

      await expect(service.refreshTokens(userId, refreshToken)).rejects.toThrow(
        UnauthorizedException,
      );
    });

    it('should throw UnauthorizedException if token is not a refresh token', async () => {
      const userId = 'user1';
      const refreshToken = 'not-a-refresh-token';
      const payload = { sub: userId }; // Missing isRefreshToken: true

      mockJwtService.verifyAsync.mockResolvedValue(payload);

      await expect(service.refreshTokens(userId, refreshToken)).rejects.toThrow(
        UnauthorizedException,
      );
    });

    it('should throw UnauthorizedException if user ID does not match', async () => {
      const userId = 'user1';
      const refreshToken = 'wrong-user-token';
      const payload = { sub: 'user2', isRefreshToken: true }; // Different user ID

      mockJwtService.verifyAsync.mockResolvedValue(payload);

      await expect(service.refreshTokens(userId, refreshToken)).rejects.toThrow(
        UnauthorizedException,
      );
    });
  });

  describe('validateJwtUser', () => {
    it('should return user if user exists', async () => {
      const userId = 'user1';
      const user = { id: userId, name: 'Test User' };
      const payload = { sub: userId };

      mockUsersService.findById.mockResolvedValue(user);

      const result = await service.validateJwtUser(payload);

      expect(mockUsersService.findById).toHaveBeenCalledWith(userId);
      expect(result).toEqual(user);
    });

    it('should throw UnauthorizedException if user does not exist', async () => {
      const userId = 'nonexistent';
      const payload = { sub: userId };

      mockUsersService.findById.mockResolvedValue(null);

      await expect(service.validateJwtUser(payload)).rejects.toThrow(
        UnauthorizedException,
      );
    });
  });
});