import {
  Controller,
  Get,
  Post,
  Body,
  Patch,
  Param,
  Delete,
  HttpCode,
  HttpStatus,
  Query,
} from '@nestjs/common';
import { PersonsService } from '../services/persons.service';
import { CreatePersonDto } from '../dto/create-person.dto';
import { UpdatePersonDto } from '../dto/update-person.dto';

@Controller('persons')
export class PersonsController {
  constructor(private readonly personsService: PersonsService) {}

  /**
   * Create a new person
   */
  @Post()
  @HttpCode(HttpStatus.CREATED)
  create(@Body() createPersonDto: CreatePersonDto) {
    return this.personsService.create(createPersonDto);
  }

  /**
   * Get all persons or filter by project ID
   */
  @Get()
  findAll(@Query('projectId') projectId?: string) {
    if (projectId) {
      return this.personsService.findByProjectId(projectId);
    }
    return this.personsService.findAll();
  }

  /**
   * Get a person by ID
   */
  @Get(':id')
  findOne(@Param('id') id: string) {
    return this.personsService.findById(id);
  }

  /**
   * Update a person
   */
  @Patch(':id')
  update(@Param('id') id: string, @Body() updatePersonDto: UpdatePersonDto) {
    return this.personsService.update(id, updatePersonDto);
  }

  /**
   * Delete a person
   */
  @Delete(':id')
  @HttpCode(HttpStatus.NO_CONTENT)
  remove(@Param('id') id: string) {
    return this.personsService.remove(id);
  }

  /**
   * Get persons by project ID and group ID
   */
  @Get('project/:projectId/group/:groupId')
  findByProjectIdAndGroupId(
    @Param('projectId') projectId: string,
    @Param('groupId') groupId: string,
  ) {
    return this.personsService.findByProjectIdAndGroupId(projectId, groupId);
  }

  /**
   * Add a person to a group
   */
  @Post(':id/groups/:groupId')
  @HttpCode(HttpStatus.CREATED)
  addToGroup(@Param('id') id: string, @Param('groupId') groupId: string) {
    return this.personsService.addToGroup(id, groupId);
  }

  /**
   * Remove a person from a group
   */
  @Delete(':id/groups/:groupId')
  @HttpCode(HttpStatus.NO_CONTENT)
  removeFromGroup(@Param('id') id: string, @Param('groupId') groupId: string) {
    return this.personsService.removeFromGroup(id, groupId);
  }
}