import { Test, TestingModule } from '@nestjs/testing';
import { TagsController } from './tags.controller';
import { TagsService } from '../services/tags.service';
import { CreateTagDto } from '../dto/create-tag.dto';
import { UpdateTagDto } from '../dto/update-tag.dto';
import { JwtAuthGuard } from '../../auth/guards/jwt-auth.guard';

describe('TagsController', () => {
  let controller: TagsController;
  let service: TagsService;

  // Mock data
  const mockTag = {
    id: 'tag1',
    name: 'Test Tag',
    description: 'Test Description',
    color: '#FF0000',
    type: 'PERSON',
    createdAt: new Date(),
    updatedAt: new Date(),
  };

  const mockPersonToTag = {
    personId: 'person1',
    tagId: 'tag1',
  };

  const mockProjectToTag = {
    projectId: 'project1',
    tagId: 'tag1',
  };

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      controllers: [TagsController],
      providers: [
        {
          provide: TagsService,
          useValue: {
            create: jest.fn().mockResolvedValue(mockTag),
            findAll: jest.fn().mockResolvedValue([mockTag]),
            findByType: jest.fn().mockResolvedValue([mockTag]),
            findById: jest.fn().mockResolvedValue(mockTag),
            update: jest.fn().mockResolvedValue(mockTag),
            remove: jest.fn().mockResolvedValue(mockTag),
            addTagToPerson: jest.fn().mockResolvedValue(mockPersonToTag),
            removeTagFromPerson: jest.fn().mockResolvedValue(mockPersonToTag),
            getTagsForPerson: jest.fn().mockResolvedValue([{ tag: mockTag }]),
            addTagToProject: jest.fn().mockResolvedValue(mockProjectToTag),
            removeTagFromProject: jest.fn().mockResolvedValue(mockProjectToTag),
            getTagsForProject: jest.fn().mockResolvedValue([{ tag: mockTag }]),
          },
        },
      ],
    })
    .overrideGuard(JwtAuthGuard)
    .useValue({ canActivate: () => true })
    .compile();

    controller = module.get<TagsController>(TagsController);
    service = module.get<TagsService>(TagsService);
  });

  it('should be defined', () => {
    expect(controller).toBeDefined();
  });

  describe('create', () => {
    it('should create a new tag', async () => {
      const createTagDto: CreateTagDto = {
        name: 'Test Tag',
        color: '#FF0000',
        type: 'PERSON',
      };

      expect(await controller.create(createTagDto)).toBe(mockTag);
      expect(service.create).toHaveBeenCalledWith(createTagDto);
    });
  });

  describe('findAll', () => {
    it('should return all tags when no type is provided', async () => {
      expect(await controller.findAll()).toEqual([mockTag]);
      expect(service.findAll).toHaveBeenCalled();
    });

    it('should return tags filtered by type when type is provided', async () => {
      const type = 'PERSON';
      expect(await controller.findAll(type)).toEqual([mockTag]);
      expect(service.findByType).toHaveBeenCalledWith(type);
    });
  });

  describe('findOne', () => {
    it('should return a tag by ID', async () => {
      const id = 'tag1';
      expect(await controller.findOne(id)).toBe(mockTag);
      expect(service.findById).toHaveBeenCalledWith(id);
    });
  });

  describe('update', () => {
    it('should update a tag', async () => {
      const id = 'tag1';
      const updateTagDto: UpdateTagDto = {
        name: 'Updated Tag',
      };

      expect(await controller.update(id, updateTagDto)).toBe(mockTag);
      expect(service.update).toHaveBeenCalledWith(id, updateTagDto);
    });
  });

  describe('remove', () => {
    it('should delete a tag', async () => {
      const id = 'tag1';
      expect(await controller.remove(id)).toBe(mockTag);
      expect(service.remove).toHaveBeenCalledWith(id);
    });
  });

  describe('addTagToPerson', () => {
    it('should add a tag to a person', async () => {
      const personId = 'person1';
      const tagId = 'tag1';

      expect(await controller.addTagToPerson(personId, tagId)).toBe(mockPersonToTag);
      expect(service.addTagToPerson).toHaveBeenCalledWith(tagId, personId);
    });
  });

  describe('removeTagFromPerson', () => {
    it('should remove a tag from a person', async () => {
      const personId = 'person1';
      const tagId = 'tag1';

      expect(await controller.removeTagFromPerson(personId, tagId)).toBe(mockPersonToTag);
      expect(service.removeTagFromPerson).toHaveBeenCalledWith(tagId, personId);
    });
  });

  describe('getTagsForPerson', () => {
    it('should get all tags for a person', async () => {
      const personId = 'person1';

      expect(await controller.getTagsForPerson(personId)).toEqual([{ tag: mockTag }]);
      expect(service.getTagsForPerson).toHaveBeenCalledWith(personId);
    });
  });

  describe('addTagToProject', () => {
    it('should add a tag to a project', async () => {
      const projectId = 'project1';
      const tagId = 'tag1';

      expect(await controller.addTagToProject(projectId, tagId)).toBe(mockProjectToTag);
      expect(service.addTagToProject).toHaveBeenCalledWith(tagId, projectId);
    });
  });

  describe('removeTagFromProject', () => {
    it('should remove a tag from a project', async () => {
      const projectId = 'project1';
      const tagId = 'tag1';

      expect(await controller.removeTagFromProject(projectId, tagId)).toBe(mockProjectToTag);
      expect(service.removeTagFromProject).toHaveBeenCalledWith(tagId, projectId);
    });
  });

  describe('getTagsForProject', () => {
    it('should get all tags for a project', async () => {
      const projectId = 'project1';

      expect(await controller.getTagsForProject(projectId)).toEqual([{ tag: mockTag }]);
      expect(service.getTagsForProject).toHaveBeenCalledWith(projectId);
    });
  });
});
