import { Test, TestingModule } from '@nestjs/testing';
import { UsersController } from './users.controller';
import { UsersService } from '../services/users.service';
import { CreateUserDto } from '../dto/create-user.dto';
import { UpdateUserDto } from '../dto/update-user.dto';
import { JwtAuthGuard } from '../../auth/guards/jwt-auth.guard';

describe('UsersController', () => {
  let controller: UsersController;
  let service: UsersService;

  // Mock data
  const mockUser = {
    id: 'user1',
    name: 'Test User',
    avatar: 'https://example.com/avatar.jpg',
    githubId: '12345',
    metadata: {},
    gdprTimestamp: new Date(),
    createdAt: new Date(),
    updatedAt: new Date(),
  };

  const mockUserData = {
    user: mockUser,
    projects: [
      {
        id: 'project1',
        name: 'Test Project',
        ownerId: 'user1',
      },
    ],
  };

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      controllers: [UsersController],
      providers: [
        {
          provide: UsersService,
          useValue: {
            create: jest.fn().mockResolvedValue(mockUser),
            findAll: jest.fn().mockResolvedValue([mockUser]),
            findById: jest.fn().mockResolvedValue(mockUser),
            update: jest.fn().mockResolvedValue(mockUser),
            remove: jest.fn().mockResolvedValue(mockUser),
            updateGdprConsent: jest.fn().mockResolvedValue(mockUser),
            exportUserData: jest.fn().mockResolvedValue(mockUserData),
          },
        },
      ],
    })
    .overrideGuard(JwtAuthGuard)
    .useValue({ canActivate: () => true })
    .compile();

    controller = module.get<UsersController>(UsersController);
    service = module.get<UsersService>(UsersService);
  });

  it('should be defined', () => {
    expect(controller).toBeDefined();
  });

  describe('create', () => {
    it('should create a new user', async () => {
      const createUserDto: CreateUserDto = {
        name: 'Test User',
        githubId: '12345',
      };

      expect(await controller.create(createUserDto)).toBe(mockUser);
      expect(service.create).toHaveBeenCalledWith(createUserDto);
    });
  });

  describe('findAll', () => {
    it('should return all users', async () => {
      expect(await controller.findAll()).toEqual([mockUser]);
      expect(service.findAll).toHaveBeenCalled();
    });
  });

  describe('findOne', () => {
    it('should return a user by ID', async () => {
      const id = 'user1';
      expect(await controller.findOne(id)).toBe(mockUser);
      expect(service.findById).toHaveBeenCalledWith(id);
    });
  });

  describe('update', () => {
    it('should update a user', async () => {
      const id = 'user1';
      const updateUserDto: UpdateUserDto = {
        name: 'Updated User',
      };

      expect(await controller.update(id, updateUserDto)).toBe(mockUser);
      expect(service.update).toHaveBeenCalledWith(id, updateUserDto);
    });
  });

  describe('remove', () => {
    it('should delete a user', async () => {
      const id = 'user1';
      expect(await controller.remove(id)).toBe(mockUser);
      expect(service.remove).toHaveBeenCalledWith(id);
    });
  });

  describe('updateGdprConsent', () => {
    it('should update GDPR consent timestamp', async () => {
      const id = 'user1';
      expect(await controller.updateGdprConsent(id)).toBe(mockUser);
      expect(service.updateGdprConsent).toHaveBeenCalledWith(id);
    });
  });

  describe('exportUserData', () => {
    it('should export user data', async () => {
      const id = 'user1';
      expect(await controller.exportUserData(id)).toBe(mockUserData);
      expect(service.exportUserData).toHaveBeenCalledWith(id);
    });
  });
});