import { Test, TestingModule } from '@nestjs/testing';
import { WebSocketsGateway } from './websockets.gateway';
import { Server, Socket } from 'socket.io';
import { Logger } from '@nestjs/common';

describe('WebSocketsGateway', () => {
  let gateway: WebSocketsGateway;
  let mockServer: Partial<Server>;
  let mockSocket: Partial<Socket>;
  let mockLogger: Partial<Logger>;
  let mockRoom: any;

  beforeEach(async () => {
    // Create mock for Socket.IO Server
    mockRoom = {
      emit: jest.fn(),
    };

    mockServer = {
      to: jest.fn().mockReturnValue(mockRoom),
    };

    // Create mock for Socket
    mockSocket = {
      id: 'socket1',
      handshake: {
        query: {
          userId: 'user1',
        },
        headers: {},
        time: new Date().toString(),
        address: '127.0.0.1',
        xdomain: false,
        secure: false,
        issued: Date.now(),
        url: '/socket.io/',
        auth: {},
      },
      join: jest.fn(),
      leave: jest.fn(),
    };

    // Create mock for Logger
    mockLogger = {
      log: jest.fn(),
      warn: jest.fn(),
    };

    const module: TestingModule = await Test.createTestingModule({
      providers: [
        WebSocketsGateway,
      ],
    }).compile();

    gateway = module.get<WebSocketsGateway>(WebSocketsGateway);

    // Manually set the server and logger properties
    gateway['server'] = mockServer as Server;
    gateway['logger'] = mockLogger as Logger;
  });

  it('should be defined', () => {
    expect(gateway).toBeDefined();
  });

  describe('afterInit', () => {
    it('should log initialization message', () => {
      gateway.afterInit(mockServer as Server);

      expect(mockLogger.log).toHaveBeenCalledWith('WebSocket Gateway initialized');
    });
  });

  describe('handleConnection', () => {
    it('should add client to connected clients and join user room if userId is provided', () => {
      gateway.handleConnection(mockSocket as Socket);

      // Check if client was added to connected clients
      expect(gateway['connectedClients'].get('socket1')).toBe('user1');

      // Check if client joined user room
      expect(mockSocket.join).toHaveBeenCalledWith('user:user1');

      // Check if connection was logged
      expect(mockLogger.log).toHaveBeenCalledWith('Client connected: socket1, User ID: user1');
    });

    it('should log warning if userId is not provided', () => {
      const socketWithoutUserId = {
        ...mockSocket,
        handshake: {
          ...mockSocket.handshake,
          query: {},
        },
      };

      gateway.handleConnection(socketWithoutUserId as Socket);

      // Check if warning was logged
      expect(mockLogger.warn).toHaveBeenCalledWith('Client connected without user ID: socket1');

      // Check if client was not added to connected clients
      expect(gateway['connectedClients'].has('socket1')).toBe(false);

      // Check if client did not join user room
      expect(mockSocket.join).not.toHaveBeenCalled();
    });
  });

  describe('handleDisconnect', () => {
    it('should remove client from connected clients', () => {
      // First add client to connected clients
      gateway['connectedClients'].set('socket1', 'user1');

      gateway.handleDisconnect(mockSocket as Socket);

      // Check if client was removed from connected clients
      expect(gateway['connectedClients'].has('socket1')).toBe(false);

      // Check if disconnection was logged
      expect(mockLogger.log).toHaveBeenCalledWith('Client disconnected: socket1');
    });
  });

  describe('handleJoinProject', () => {
    it('should join project room and return success', () => {
      const projectId = 'project1';

      const result = gateway.handleJoinProject(mockSocket as Socket, projectId);

      // Check if client joined project room
      expect(mockSocket.join).toHaveBeenCalledWith('project:project1');

      // Check if join was logged
      expect(mockLogger.log).toHaveBeenCalledWith('Client socket1 joined project room: project1');

      // Check if success was returned
      expect(result).toEqual({ success: true });
    });
  });

  describe('handleLeaveProject', () => {
    it('should leave project room and return success', () => {
      const projectId = 'project1';

      const result = gateway.handleLeaveProject(mockSocket as Socket, projectId);

      // Check if client left project room
      expect(mockSocket.leave).toHaveBeenCalledWith('project:project1');

      // Check if leave was logged
      expect(mockLogger.log).toHaveBeenCalledWith('Client socket1 left project room: project1');

      // Check if success was returned
      expect(result).toEqual({ success: true });
    });
  });

  describe('emitProjectUpdated', () => {
    it('should emit project:updated event to project room', () => {
      const projectId = 'project1';
      const data = { action: 'updated', project: { id: projectId } };

      gateway.emitProjectUpdated(projectId, data);

      // Check if event was emitted to project room
      expect(mockServer.to).toHaveBeenCalledWith('project:project1');
      expect(mockRoom.emit).toHaveBeenCalledWith('project:updated', data);

      // Check if emit was logged
      expect(mockLogger.log).toHaveBeenCalledWith('Emitted project:updated for project project1');
    });
  });

  describe('emitCollaboratorAdded', () => {
    it('should emit project:collaboratorAdded event to project room', () => {
      const projectId = 'project1';
      const data = { project: { id: projectId }, user: { id: 'user1' } };

      gateway.emitCollaboratorAdded(projectId, data);

      // Check if event was emitted to project room
      expect(mockServer.to).toHaveBeenCalledWith('project:project1');
      expect(mockRoom.emit).toHaveBeenCalledWith('project:collaboratorAdded', data);

      // Check if emit was logged
      expect(mockLogger.log).toHaveBeenCalledWith('Emitted project:collaboratorAdded for project project1');
    });
  });

  describe('emitGroupCreated', () => {
    it('should emit group:created event to project room', () => {
      const projectId = 'project1';
      const data = { action: 'created', group: { id: 'group1' } };

      gateway.emitGroupCreated(projectId, data);

      // Check if event was emitted to project room
      expect(mockServer.to).toHaveBeenCalledWith('project:project1');
      expect(mockRoom.emit).toHaveBeenCalledWith('group:created', data);

      // Check if emit was logged
      expect(mockLogger.log).toHaveBeenCalledWith('Emitted group:created for project project1');
    });
  });

  describe('emitGroupUpdated', () => {
    it('should emit group:updated event to project room', () => {
      const projectId = 'project1';
      const data = { action: 'updated', group: { id: 'group1' } };

      gateway.emitGroupUpdated(projectId, data);

      // Check if event was emitted to project room
      expect(mockServer.to).toHaveBeenCalledWith('project:project1');
      expect(mockRoom.emit).toHaveBeenCalledWith('group:updated', data);

      // Check if emit was logged
      expect(mockLogger.log).toHaveBeenCalledWith('Emitted group:updated for project project1');
    });
  });

  describe('emitPersonAddedToGroup', () => {
    it('should emit group:personAdded event to project room', () => {
      const projectId = 'project1';
      const data = { group: { id: 'group1' }, person: { id: 'person1' } };

      gateway.emitPersonAddedToGroup(projectId, data);

      // Check if event was emitted to project room
      expect(mockServer.to).toHaveBeenCalledWith('project:project1');
      expect(mockRoom.emit).toHaveBeenCalledWith('group:personAdded', data);

      // Check if emit was logged
      expect(mockLogger.log).toHaveBeenCalledWith('Emitted group:personAdded for project project1');
    });
  });

  describe('emitPersonRemovedFromGroup', () => {
    it('should emit group:personRemoved event to project room', () => {
      const projectId = 'project1';
      const data = { group: { id: 'group1' }, person: { id: 'person1' } };

      gateway.emitPersonRemovedFromGroup(projectId, data);

      // Check if event was emitted to project room
      expect(mockServer.to).toHaveBeenCalledWith('project:project1');
      expect(mockRoom.emit).toHaveBeenCalledWith('group:personRemoved', data);

      // Check if emit was logged
      expect(mockLogger.log).toHaveBeenCalledWith('Emitted group:personRemoved for project project1');
    });
  });

  describe('emitNotification', () => {
    it('should emit notification:new event to user room', () => {
      const userId = 'user1';
      const data = { type: 'info', message: 'Test notification' };

      gateway.emitNotification(userId, data);

      // Check if event was emitted to user room
      expect(mockServer.to).toHaveBeenCalledWith('user:user1');
      expect(mockRoom.emit).toHaveBeenCalledWith('notification:new', data);

      // Check if emit was logged
      expect(mockLogger.log).toHaveBeenCalledWith('Emitted notification:new for user user1');
    });
  });

  describe('emitProjectNotification', () => {
    it('should emit notification:new event to project room', () => {
      const projectId = 'project1';
      const data = { type: 'info', message: 'Test project notification' };

      gateway.emitProjectNotification(projectId, data);

      // Check if event was emitted to project room
      expect(mockServer.to).toHaveBeenCalledWith('project:project1');
      expect(mockRoom.emit).toHaveBeenCalledWith('notification:new', data);

      // Check if emit was logged
      expect(mockLogger.log).toHaveBeenCalledWith('Emitted notification:new for project project1');
    });
  });
});
