"use client";

import { useState, useEffect } from "react";
import { Badge } from "@/components/ui/badge";
import { Button } from "@/components/ui/button";
import { Command, CommandEmpty, CommandGroup, CommandInput, CommandItem } from "@/components/ui/command";
import { Popover, PopoverContent, PopoverTrigger } from "@/components/ui/popover";
import { Check, ChevronsUpDown, X } from "lucide-react";
import { cn } from "@/lib/utils";

// Map color names to Tailwind classes
const colorMap: Record<string, string> = {
  blue: "bg-blue-100 text-blue-800 dark:bg-blue-900 dark:text-blue-300",
  green: "bg-green-100 text-green-800 dark:bg-green-900 dark:text-green-300",
  purple: "bg-purple-100 text-purple-800 dark:bg-purple-900 dark:text-purple-300",
  pink: "bg-pink-100 text-pink-800 dark:bg-pink-900 dark:text-pink-300",
  orange: "bg-orange-100 text-orange-800 dark:bg-orange-900 dark:text-orange-300",
  yellow: "bg-yellow-100 text-yellow-800 dark:bg-yellow-900 dark:text-yellow-300",
  amber: "bg-amber-100 text-amber-800 dark:bg-amber-900 dark:text-amber-300",
  red: "bg-red-100 text-red-800 dark:bg-red-900 dark:text-red-300",
};

export interface Tag {
  id: number;
  name: string;
  description: string;
  color: string;
}

interface TagSelectorProps {
  selectedTags: Tag[];
  onChange: (tags: Tag[]) => void;
  placeholder?: string;
  disabled?: boolean;
  className?: string;
}

export function TagSelector({
  selectedTags = [],
  onChange,
  placeholder = "Sélectionner des tags...",
  disabled = false,
  className,
}: TagSelectorProps) {
  const [open, setOpen] = useState(false);
  const [tags, setTags] = useState<Tag[]>([]);
  const [loading, setLoading] = useState(true);

  // Mock data for tags - in a real app, this would be fetched from an API
  useEffect(() => {
    const fetchTags = async () => {
      setLoading(true);
      try {
        // Simulate API call
        await new Promise(resolve => setTimeout(resolve, 500));
        
        // Mock data
        const mockTags = [
          {
            id: 1,
            name: "Frontend",
            description: "Développement frontend",
            color: "blue",
          },
          {
            id: 2,
            name: "Backend",
            description: "Développement backend",
            color: "green",
          },
          {
            id: 3,
            name: "Fullstack",
            description: "Développement fullstack",
            color: "purple",
          },
          {
            id: 4,
            name: "UX/UI",
            description: "Design UX/UI",
            color: "pink",
          },
          {
            id: 5,
            name: "DevOps",
            description: "Infrastructure et déploiement",
            color: "orange",
          },
          {
            id: 6,
            name: "Junior",
            description: "Niveau junior",
            color: "yellow",
          },
          {
            id: 7,
            name: "Medior",
            description: "Niveau intermédiaire",
            color: "amber",
          },
          {
            id: 8,
            name: "Senior",
            description: "Niveau senior",
            color: "red",
          },
        ];
        
        setTags(mockTags);
      } catch (error) {
        console.error("Error fetching tags:", error);
      } finally {
        setLoading(false);
      }
    };

    fetchTags();
  }, []);

  const handleSelect = (tag: Tag) => {
    const isSelected = selectedTags.some(t => t.id === tag.id);
    
    if (isSelected) {
      onChange(selectedTags.filter(t => t.id !== tag.id));
    } else {
      onChange([...selectedTags, tag]);
    }
  };

  const handleRemove = (tagId: number) => {
    onChange(selectedTags.filter(tag => tag.id !== tagId));
  };

  return (
    <div className={cn("space-y-2", className)}>
      <Popover open={open} onOpenChange={setOpen}>
        <PopoverTrigger asChild>
          <Button
            variant="outline"
            role="combobox"
            aria-expanded={open}
            className="w-full justify-between"
            disabled={disabled || loading}
          >
            {selectedTags.length > 0
              ? `${selectedTags.length} tag${selectedTags.length > 1 ? "s" : ""} sélectionné${selectedTags.length > 1 ? "s" : ""}`
              : placeholder}
            <ChevronsUpDown className="ml-2 h-4 w-4 shrink-0 opacity-50" />
          </Button>
        </PopoverTrigger>
        <PopoverContent className="w-full p-0">
          <Command>
            <CommandInput placeholder="Rechercher un tag..." />
            <CommandEmpty>Aucun tag trouvé.</CommandEmpty>
            <CommandGroup className="max-h-64 overflow-auto">
              {loading ? (
                <div className="flex items-center justify-center p-4">
                  <span className="h-4 w-4 animate-spin rounded-full border-2 border-primary border-t-transparent" />
                </div>
              ) : (
                tags.map(tag => (
                  <CommandItem
                    key={tag.id}
                    value={tag.name}
                    onSelect={() => handleSelect(tag)}
                  >
                    <Check
                      className={cn(
                        "mr-2 h-4 w-4",
                        selectedTags.some(t => t.id === tag.id)
                          ? "opacity-100"
                          : "opacity-0"
                      )}
                    />
                    <div className="flex items-center gap-2">
                      <Badge className={colorMap[tag.color]}>
                        {tag.name}
                      </Badge>
                      <span className="text-sm text-muted-foreground">
                        {tag.description}
                      </span>
                    </div>
                  </CommandItem>
                ))
              )}
            </CommandGroup>
          </Command>
        </PopoverContent>
      </Popover>

      {selectedTags.length > 0 && (
        <div className="flex flex-wrap gap-2 mt-2">
          {selectedTags.map(tag => (
            <Badge
              key={tag.id}
              className={cn(
                colorMap[tag.color],
                "flex items-center gap-1 pr-1"
              )}
            >
              {tag.name}
              <Button
                variant="ghost"
                size="icon"
                className="h-4 w-4 rounded-full p-0 hover:bg-background/20"
                onClick={() => handleRemove(tag.id)}
                disabled={disabled}
              >
                <X className="h-3 w-3" />
                <span className="sr-only">Supprimer</span>
              </Button>
            </Badge>
          ))}
        </div>
      )}
    </div>
  );
}